#pragma once

#include <string>
#include <memory>
#include <vector>
#include "domain/Workflow.h"
#include "domain/ProcessEngine.h"

class ProcessManager {

public:
  ProcessManager(std::string basePath, std::string workflowDirectory, std::string engineFile, std::string statusFile);

  // main commands / functionality
  bool startWorkflow(const std::string& workflowFile, const std::string& workflowEngine);
  bool continueWorkflow(unsigned int workflowId);
  bool listWorkflows() const;
  bool listEngines() const;
  bool printWorkflowLog(unsigned int workflowId);
  bool inputInteractionValue(unsigned int workflowId, std::string interactionId, const std::string& inputValue);
  bool printWorkflowStatus(unsigned int workflowId);
  bool printWorkflowShortcuts(unsigned int workflowId);
  bool printWorkflowInteractions(unsigned int workflowId);

  // public interface for other classes
  void log(const std::string& message) const;
  void error(const std::string& message) const;

private:
  void loadEngines();

  void loadWorkflows();
  void saveWorkflows() const;

  const ProcessEngine *getProcessEngine(const std::string &engineName, bool exitOnError, bool logErrors = true) const;
  Workflow *getExistingWorkflow(unsigned int workflowId, bool exitOnError, bool logErrors = true) const;


  json retrieveWorkflowStatus(unsigned int id) const;
  std::string getWorkflowExecutionFolder(unsigned int id) const;

  std::string basePath;
  std::string workflowsDirectory;
  std::string engineFile;
  std::string workflowsFile;

  std::string defaultProcessEngine;
  int nextId;
  std::unordered_map<unsigned int, std::unique_ptr<Workflow>> workflows;
  std::unordered_map<std::string, std::unique_ptr<ProcessEngine>> processEngines;

  bool updateWorkflow(Workflow *workflow) const;
};
