

#include "Workflow.h"

#include <utility>
#include <iostream>
#include <iomanip>


Workflow::Workflow(int id, std::string fileName, WorkflowState state, std::string startDateTime, std::string endDateTime,
                   std::string processEngine, int nodesProcessed, int nodesTotal)
  : id(id), fileName(std::move(fileName)), state(state), startDateTime(std::move(startDateTime)),
    endDateTime(std::move(endDateTime)), processEngine(std::move(processEngine)), nodesProcessed(nodesProcessed),
    nodesTotal(nodesTotal) {

}

std::string Workflow::getFileName() const {
  return fileName;
}

WorkflowState Workflow::getState() const {
  return state;
}

const std::string &Workflow::getStartDateTime() const {
  return startDateTime;
}

const std::string &Workflow::getEndDateTime() const {
  return endDateTime;
}

std::string Workflow::getProcessEngine() const {
  return processEngine;
}

std::string Workflow::stateToString(WorkflowState state) {
  switch(state) {
    case READY:
      return "Ready";
    case RUNNING:
      return "Running";
    case SUSPENDED:
      return "Suspended";
    case ERROR:
      return "Error";
    case NEEDS_INTERACTION:
      return "Needs_interaction";
    case STOPPED:
      return "Stopped";
    case FINISHED:
      return "Finished";
    default:
      return "Unknown state";
  }
}

WorkflowState Workflow::stringToState(const std::string& stringState) {
  if (stringState == "Ready") {
    return READY;
  } else if (stringState == "Running") {
    return RUNNING;
  } else if (stringState == "Suspended") {
    return SUSPENDED;
  } else if (stringState == "Error") {
    return ERROR;
  } else if (stringState == "Needs_interaction") {
    return NEEDS_INTERACTION;
  } else if (stringState == "Stopped") {
    return STOPPED;
  } else if (stringState == "Finished") {
    return FINISHED;
  } else {
    throw std::logic_error("Unknown state string: " + stringState);
  }
}

json Workflow::getJson() const {
  json jsonObj;
  jsonObj["id"] = id;
  jsonObj["fileName"] = fileName;
  jsonObj["state"] = stateToString(state);
  jsonObj["startDateTime"] = startDateTime;
  jsonObj["endDateTime"] = endDateTime;
  jsonObj["processEngine"] = processEngine;
  jsonObj["nodesProcessed"] = nodesProcessed;
  jsonObj["nodesTotal"] = nodesTotal;
  return jsonObj;
}

void Workflow::fromJson(json object) {
  if (object.empty() || object == nullptr) {
    std::string error_message = "Unable to parse workflow json: ";
    std::cerr << error_message << std::setw(4) << object << std::endl;
    throw std::logic_error(error_message);
  }
  if (object.contains("id")) {
    id = object["id"].get<int>();
  }
  fileName = object["fileName"].get<std::string>();
  state = stringToState(object["state"].get<std::string>());
  startDateTime = object["startDateTime"].get<std::string>();
  endDateTime = object["endDateTime"].get<std::string>();
  processEngine = object["processEngine"].get<std::string>();
  nodesProcessed = object["nodesProcessed"].get<int>();
  nodesTotal = object["nodesTotal"].get<int>();
}

void Workflow::setId(unsigned int id) {
  Workflow::id = id;
}

unsigned int Workflow::getId() const {
  return id;
}
