#pragma once

#include <string>
#include "Serializable.h"

enum WorkflowState {
  READY,
  RUNNING,
  SUSPENDED,
  ERROR,
  NEEDS_INTERACTION,
  STOPPED,
  FINISHED
};

class Workflow : public Serializable {

public:
  Workflow() = default;
  Workflow(int id, std::string fileName, WorkflowState state, std::string startDateTime, std::string endDateTime,
           std::string processEngine, int nodesProcessed, int nodesTotal);

  std::string getFileName() const;
  WorkflowState getState() const;
  const std::string &getStartDateTime() const;
  const std::string &getEndDateTime() const;
  std::string getProcessEngine() const;
  unsigned int getId() const;
  void setId(unsigned int id);

  static std::string stateToString(WorkflowState state);
  static WorkflowState stringToState(const std::string& stringState);

  json getJson() const override;
  void fromJson(json object) override ;

private:
  unsigned int id;
  std::string fileName;
  WorkflowState state;
  std::string startDateTime;
  std::string endDateTime;
  std::string processEngine;
  int nodesProcessed;
  int nodesTotal;
};
