#include <stdexcept>
#include <experimental/filesystem>
#include <iostream>

namespace fs = std::experimental::filesystem;

namespace util {

  bool fileExists(const std::string& fileName) {
    return fs::exists(fileName);
  }

  bool isAbsolutePath(const std::string& path) {
    fs::path toCheck = path;
    return toCheck.is_absolute();
  }

  std::string getAbsolutePath(const std::string& fileName) {
    return fs::absolute(fileName);
  }

  bool createDirectory(const std::string& path) {
    bool result = false;

    if (!fs::is_directory(path) || !fs::exists(path)) {
      try {
        result = fs::create_directories(path); // creates non-existing parents
      } catch(fs::filesystem_error& error) {
        std::cerr << error.what() << std::endl;
        result = false;
      }
    }
    return result;
  }

  bool copyFile(const std::string& from, const std::string& to) {
    return fs::copy_file(from, to);
  }
}
