#pragma once

#include <string>

namespace util {

  /**
   * Check if given file exists on the file system
   * @param fileName Name (incl. path) to check
   * @return true if the file exists
   */
  bool fileExists(const std::string& fileName);


  /** Check if a path is absolute
   * @param path path to be checked
   * @return true if path is absolute
   */
  bool isAbsolutePath(const std::string& path);

  /**
   * Get absolute path for a filename in the same directory (adding cwd)
   * @param fileName
   * @return absolute path
   */
  std::string getAbsolutePath(const std::string& fileName);

  /**
   * Creates the specified directory if it does not exist
   * Also creates non-existent parents (like `mkdir -p`)
   * @param path Path of the directory to create
   * @return true if the directory exists afterwards (was created or existed before)
   */
  bool createDirectory(const std::string& path);

  /**
   * Copies a file to a destination path
   * @param path Path of the file to copy
   * @param path Destination path
   * @return true if the file was copied successfully
   */
  bool copyFile(const std::string& from, const std::string& to);
}

